module Nominal where

import Atom
import AbsM


class Nominal t where
    swap                         :: Atom -> Atom -> t -> t
    fresh                        :: Atom -> t -> Bool
    supp                         :: t -> [Atom] -- TODO : Atom set


instance Nominal Atom where
    swap a b c                   = if c == a then b 
				   else if c == b then a
				   else c
    fresh a b                    = not(a == b)
    supp a                       = [a]

instance Nominal Int where
    swap a b c                   = c
    fresh a c                    = True
    supp c                       = []

instance Nominal Char where
    swap a b c                   = c
    fresh a c                    = True
    supp c                       = []

instance Nominal () where
    swap a b ()                  = ()
    fresh a ()                   = True
    supp ()                      = []

instance (Nominal t) => Nominal [t] where
    swap a b l                   = map (swap a b) l
    fresh a l                    = foldr (\t b -> b && fresh a t) True l
    supp t                       = foldr (++) [] (map supp t) 




instance Nominal t => Nominal (Maybe t) where
    swap a b t                   = swap0 a b t
				     where swap0 a b Nothing = Nothing
					   swap0 a b (Just t) = Just (swap a b t)
    fresh a Nothing              = True
    fresh a (Just t)             = fresh a t
    supp Nothing                 = []
    supp (Just t)                = supp t

instance (Nominal t1, Nominal t2) => Nominal (t1,t2) where
    swap a b (t1,t2)             = (swap a b t1, swap a b t2)
    fresh a (t1,t2)              = fresh a t1 && fresh a t2
    supp (t1,t2)                 = supp t1 ++ supp t2

instance (Nominal t1, Nominal t2) => Nominal (Either t1 t2) where
    swap a b (Left t1)           = Left (swap a b t1)
    swap a b (Right t2)          = Right (swap a b t2)
    fresh a (Left t)             = fresh a t
    fresh a (Right t)            = fresh a t
    supp (Left t)                = supp t
    supp (Right t)               = supp t
